@@ -29,6 +29,8 @@ class ModelsTest(parameterized.TestCase):
       dict(testcase_name="one_channel", shape=(10, 1)),
       dict(testcase_name="five_channel", shape=(10, 5)),
       dict(testcase_name="same_channels_as_rows", shape=(10, 10)),
+      dict(testcase_name="geo_shape_1", shape=(10, 10, 5)),
+      dict(testcase_name="geo_shape_2", shape=(10, 5, 2)),
       dict(testcase_name="one_channel_one_row", shape=(1, 1)))
   def test_transform_adstock_produces_correct_output_shape(self, shape):
 
@@ -39,7 +41,7 @@ class ModelsTest(parameterized.TestCase):
     media = jnp.ones(shape)
     kernel = numpyro.infer.NUTS(model=mock_model_function)
     mcmc = numpyro.infer.MCMC(
-        sampler=kernel, num_warmup=100, num_samples=10, num_chains=1)
+        sampler=kernel, num_warmup=10, num_samples=10, num_chains=1)
     rng_key = jax.random.PRNGKey(0)
 
     mcmc.run(rng_key, media_data=media)
@@ -51,17 +53,20 @@ class ModelsTest(parameterized.TestCase):
       dict(testcase_name="one_channel", shape=(10, 1)),
       dict(testcase_name="five_channel", shape=(10, 5)),
       dict(testcase_name="same_channels_as_rows", shape=(10, 10)),
+      dict(testcase_name="geo_shape_1", shape=(10, 10, 5)),
+      dict(testcase_name="geo_shape_2", shape=(10, 5, 2)),
       dict(testcase_name="one_channel_one_row", shape=(1, 1)))
   def test_transform_hill_adstock_produces_correct_output_shape(self, shape):
 
     def mock_model_function(media_data):
-      numpyro.deterministic("transformed_media",
-                            models.transform_hill_adstock(media_data))
+      numpyro.deterministic(
+          "transformed_media",
+          models.transform_hill_adstock(media_data))
 
     media = jnp.ones(shape)
     kernel = numpyro.infer.NUTS(model=mock_model_function)
     mcmc = numpyro.infer.MCMC(
-        sampler=kernel, num_warmup=100, num_samples=10, num_chains=1)
+        sampler=kernel, num_warmup=10, num_samples=10, num_chains=1)
     rng_key = jax.random.PRNGKey(0)
 
     mcmc.run(rng_key, media_data=media)
@@ -73,17 +78,20 @@ class ModelsTest(parameterized.TestCase):
       dict(testcase_name="one_channel", shape=(10, 1)),
       dict(testcase_name="five_channel", shape=(10, 5)),
       dict(testcase_name="same_channels_as_rows", shape=(10, 10)),
+      dict(testcase_name="geo_shape_1", shape=(10, 10, 5)),
+      dict(testcase_name="geo_shape_2", shape=(10, 5, 2)),
       dict(testcase_name="one_channel_one_row", shape=(1, 1)))
   def test_transform_carryover_produces_correct_output_shape(self, shape):
 
     def mock_model_function(media_data):
-      numpyro.deterministic("transformed_media",
-                            models.transform_carryover(media_data))
+      numpyro.deterministic(
+          "transformed_media",
+          models.transform_carryover(media_data))
 
     media = jnp.ones(shape)
     kernel = numpyro.infer.NUTS(model=mock_model_function)
     mcmc = numpyro.infer.MCMC(
-        sampler=kernel, num_warmup=100, num_samples=10, num_chains=1)
+        sampler=kernel, num_warmup=10, num_samples=10, num_chains=1)
     rng_key = jax.random.PRNGKey(0)
 
     mcmc.run(rng_key, media_data=media)
@@ -92,19 +100,41 @@ class ModelsTest(parameterized.TestCase):
     self.assertEqual(media.shape, transformed_media.shape)
 
   @parameterized.named_parameters(
-      dict(testcase_name="one_channel", shape=(10, 1)),
-      dict(testcase_name="five_channel", shape=(10, 5)),
-      dict(testcase_name="same_channels_as_rows", shape=(10, 10)),
-      dict(testcase_name="one_channel_one_row", shape=(1, 1)))
-  def test_media_mix_model_parameters_have_correct_shapes(self, shape):
-    media = jnp.ones(shape)
-    extra_features = jnp.ones((media.shape[0], 2))
-    costs_prior = media.sum(axis=0)
+      dict(
+          testcase_name="national_no_extra",
+          media_shape=(10, 3),
+          extra_features_shape=(),
+          target_shape=(10,),
+          total_costs_shape=(3,)),
+      dict(
+          testcase_name="national_extra",
+          media_shape=(10, 5),
+          extra_features_shape=(10, 2),
+          target_shape=(10,),
+          total_costs_shape=(5,)),
+      dict(
+          testcase_name="geo_extra_3d",
+          media_shape=(10, 7, 5),
+          extra_features_shape=(10, 8, 5),
+          target_shape=(10, 5),
+          total_costs_shape=(7, 1)),
+      dict(
+          testcase_name="geo_no_extra",
+          media_shape=(10, 7, 5),
+          extra_features_shape=(),
+          target_shape=(10, 5),
+          total_costs_shape=(7, 1)))
+  def test_media_mix_model_parameters_have_correct_shapes(
+      self, media_shape, extra_features_shape, target_shape, total_costs_shape):
+    media = jnp.ones(media_shape)
+    extra_features = None if not extra_features_shape else jnp.ones(
+        extra_features_shape)
+    costs_prior = jnp.ones(total_costs_shape)
     degrees = 2
-    target = jnp.ones(shape[0])
+    target = jnp.ones(target_shape)
     kernel = numpyro.infer.NUTS(model=models.media_mix_model)
     mcmc = numpyro.infer.MCMC(
-        sampler=kernel, num_warmup=100, num_samples=10, num_chains=1)
+        sampler=kernel, num_warmup=10, num_samples=10, num_chains=1)
     rng_key = jax.random.PRNGKey(0)
 
     mcmc.run(
@@ -118,16 +148,22 @@ class ModelsTest(parameterized.TestCase):
         transform_function=models.transform_carryover)
     trace = mcmc.get_samples()
 
-    self.assertEqual(trace["intercept"].mean(axis=0).shape, ())
-    self.assertEqual(trace["sigma"].mean(axis=0).shape, ())
-    self.assertEqual(trace["beta_trend"].mean(axis=0).shape, ())
-    self.assertEqual(trace["beta_media"].mean(axis=0).shape, (shape[1],))
-    self.assertEqual(trace["beta_extra_features"].mean(axis=0).shape,
-                     (extra_features.shape[1],))
+    self.assertEqual(
+        jnp.squeeze(trace["intercept"].mean(axis=0)).shape, target_shape[1:])
+    self.assertEqual(
+        jnp.squeeze(trace["sigma"].mean(axis=0)).shape, target_shape[1:])
+    self.assertEqual(
+        jnp.squeeze(trace["expo_trend"].mean(axis=0)).shape, ())
+    self.assertEqual(
+        jnp.squeeze(trace["beta_trend"].mean(axis=0)).shape, target_shape[1:])
+    self.assertEqual(
+        jnp.squeeze(trace["beta_media"].mean(axis=0)).shape, media_shape[1:])
+    if extra_features_shape:
+      self.assertEqual(trace["beta_extra_features"].mean(axis=0).shape,
+                       extra_features.shape[1:])
     self.assertEqual(trace["gamma_seasonality"].mean(axis=0).shape,
                      (degrees, 2))
-    self.assertEqual(trace["mu"].mean(axis=0).shape, (shape[0],))
-
+    self.assertEqual(trace["mu"].mean(axis=0).shape, target_shape)
 
 if __name__ == "__main__":
   absltest.main()
