@@ -62,7 +62,7 @@ def _make_single_prediction(media_mix_model: lightweight_mmm.LightweightMMM,
 
 @functools.partial(
     jax.jit,
-    static_argnames=("media_mix_model", "extra_features", "target_scaler",
+    static_argnames=("media_mix_model", "target_scaler",
                      "apply_log_scale"))
 def _generate_diagonal_predictions(
     media_mix_model: lightweight_mmm.LightweightMMM,
@@ -96,14 +96,21 @@ def _generate_diagonal_predictions(
   """
   make_predictions = jax.vmap(fun=_make_single_prediction,
                               in_axes=(None, 0, None, None))
-  diag_media_values = jnp.eye(media_values.shape[0]) * media_values
+  diagonal = jnp.eye(media_values.shape[0])
+  if media_values.ndim == 2:  # Only two since we only provide one row
+    diagonal = jnp.expand_dims(diagonal, axis=-1)
+    media_values = jnp.expand_dims(media_values, axis=0)
+  diag_media_values = diagonal * media_values
   predictions = make_predictions(
       media_mix_model,
       diag_media_values,
       extra_features,
       seed) - prediction_offset
+  predictions = jnp.squeeze(predictions)
   if target_scaler:
     predictions = target_scaler.inverse_transform(predictions)
+  if predictions.ndim == 2:
+    predictions = jnp.sum(predictions, axis=-1)
   return predictions
 
 
@@ -128,7 +135,7 @@ def plot_response_curves(
     media_scaler: Optional[preprocessing.CustomScaler] = None,
     target_scaler: Optional[preprocessing.CustomScaler] = None,
     prices: jnp.ndarray = None,
-    optimal_allocation_per_timeunit: jnp.ndarray = None,
+    optimal_allocation_per_timeunit: Optional[jnp.ndarray] = None,
     steps: int = 50,
     percentage_add: float = 0.2,
     apply_log_scale: bool = False,
@@ -194,8 +201,10 @@ def plot_response_curves(
       jnp.linspace(start=0, stop=media_maxes, num=steps), axis=0)
 
   make_predictions = jax.vmap(
-      jax.vmap(_make_single_prediction, in_axes=(None, 0, None, None)),
-      in_axes=(None, 0, None, None))
+      jax.vmap(_make_single_prediction,
+               in_axes=(None, 0, None, None),
+               out_axes=0),
+      in_axes=(None, 0, None, None), out_axes=1)
   diagonal = jnp.repeat(
       jnp.eye(media_mix_model.n_media_channels), steps,
       axis=0).reshape(media_mix_model.n_media_channels, steps,
@@ -204,12 +213,17 @@ def plot_response_curves(
   prediction_offset = media_mix_model.predict(
       media=jnp.zeros((1, *media.shape[1:])),
       extra_features=extra_features).mean(axis=0)
+
+  if media.ndim == 3:
+    diagonal = jnp.expand_dims(diagonal, axis=-1)
+    prediction_offset = jnp.expand_dims(prediction_offset, axis=0)
   mock_media = media_ranges * diagonal
   predictions = jnp.squeeze(a=make_predictions(media_mix_model,
                                                mock_media,
                                                extra_features,
                                                seed))
-  predictions = jnp.transpose(predictions) - prediction_offset
+  predictions = predictions - prediction_offset
+  media_ranges = jnp.squeeze(media_ranges)
   if target_scaler:
     predictions = target_scaler.inverse_transform(predictions)
 
@@ -217,8 +231,14 @@ def plot_response_curves(
     media_ranges = media_scaler.inverse_transform(media_ranges)
 
   if prices is not None:
+    if media.ndim == 3:
+      prices = jnp.expand_dims(prices, axis=-1)
     media_ranges *= prices
 
+  if predictions.ndim == 3:
+    media_ranges = jnp.sum(media_ranges, axis=-1)
+    predictions = jnp.sum(predictions, axis=-1)
+
   if optimal_allocation_per_timeunit is not None:
     average_allocation = media_mix_model.media.mean(axis=0)
     average_allocation_predictions = _generate_diagonal_predictions(
@@ -235,7 +255,6 @@ def plot_response_curves(
         target_scaler=target_scaler,
         prediction_offset=prediction_offset,
         seed=seed)
-
     if media_scaler:
       average_allocation = media_scaler.inverse_transform(average_allocation)
       optimal_allocation_per_timeunit = media_scaler.inverse_transform(
@@ -243,6 +262,10 @@ def plot_response_curves(
     if prices is not None:
       optimal_allocation_per_timeunit *= prices
       average_allocation *= prices
+    if media.ndim == 3:
+      average_allocation = jnp.sum(average_allocation, axis=-1)
+      optimal_allocation_per_timeunit = jnp.sum(
+          optimal_allocation_per_timeunit, axis=-1)
 
   kpi_label = "KPI" if target_scaler else "Normalized KPI"
   fig = plt.figure(media_mix_model.n_media_channels + 1,
@@ -254,13 +277,13 @@ def plot_response_curves(
   for i in range(media_mix_model.n_media_channels):
     ax = fig.add_subplot(n_rows, n_columns, i + 1)
     sns.lineplot(
-        x=jnp.squeeze(media_ranges)[:, i],
+        x=media_ranges[:, i],
         y=predictions[:, i],
         label=media_mix_model.media_names[i],
         color=sns.color_palette()[i],
         ax=ax)
     sns.lineplot(
-        x=jnp.squeeze(media_ranges)[:, i],
+        x=media_ranges[:, i],
         y=jnp.log(predictions[:, i]) if apply_log_scale else predictions[:, i],
         label=media_mix_model.media_names[i],
         color=sns.color_palette()[i],
@@ -355,58 +378,97 @@ def plot_var_cost(media: jnp.ndarray, costs: jnp.ndarray,
   return fig
 
 
-def _create_shaded_line_plot(prediction: jnp.ndarray,
-                             true_values: jnp.ndarray,
+def _create_shaded_line_plot(predictions: jnp.ndarray,
+                             target: jnp.ndarray,
+                             axis: matplotlib.axes.Axes,
+                             title_prefix: str = "",
                              interval_mid_range: float = .9,
-                             digits: int = 3) -> matplotlib.figure.Figure:
+                             digits: int = 3) -> None:
   """Creates a plot of ground truth, predicted value and credibility interval.
 
   Args:
-    prediction: 2d array of predicted values.
-    true_values: Array of true values. Must be same length as prediction.
+    predictions: 2d array of predicted values.
+    target: Array of true values. Must be same length as predictions.
+    axis: Matplotlib axis in which to plot the data.
+    title_prefix: Prefix to add as the label of the plot.
+    interval_mid_range: Mid range interval to take for plotting. Eg. .9 will use
+      .05 and .95 as the lower and upper quantiles. Must be a float number
+      between 0 and 1.
+    digits: Number of decimals to display on metrics in the plot.
+  """
+  if predictions.shape[1] != len(target):
+    raise ValueError(
+        "Predicted data and ground-truth data must have same length.")
+  upper_quantile = 1 - (1 - interval_mid_range) / 2
+  lower_quantile = (1 - interval_mid_range) / 2
+  upper_bound = jnp.quantile(a=predictions, q=upper_quantile, axis=0)
+  lower_bound = jnp.quantile(a=predictions, q=lower_quantile, axis=0)
+
+  r2, _ = arviz.r2_score(y_true=target, y_pred=predictions)
+  mape = 100 * metrics.mean_absolute_percentage_error(
+      y_true=target, y_pred=predictions.mean(axis=0))
+  axis.plot(jnp.arange(target.shape[0]), target, c="grey", alpha=.9)
+  axis.plot(
+      jnp.arange(target.shape[0]),
+      predictions.mean(axis=0),
+      c="green",
+      alpha=.9)
+  axis.fill_between(
+      x=jnp.arange(target.shape[0]),
+      y1=lower_bound,
+      y2=upper_bound,
+      alpha=.35,
+      color="green")
+  axis.legend(["True KPI", "Predicted KPI"])
+  axis.yaxis.grid(color="gray", linestyle="dashed", alpha=0.3)
+  axis.xaxis.grid(color="gray", linestyle="dashed", alpha=0.3)
+  title = " ".join([
+      title_prefix,
+      "True and predicted KPI.",
+      "R2 = {r2:.{digits}f}".format(r2=r2, digits=digits),
+      "MAPE = {mape:.{digits}f}%".format(mape=mape, digits=digits)
+  ])
+  axis.title.set_text(title)
+  plt.close()
+
+
+def _call_fit_plotter(
+    predictions: jnp.array,
+    target: jnp.array,
+    interval_mid_range: float,
+    digits: int) -> matplotlib.figure.Figure:
+  """Calls the shaded line plot once for national and N times for geo models.
+
+  Args:
+    predictions: 2d array of predicted values.
+    target: Array of true values. Must be same length as prediction.
     interval_mid_range: Mid range interval to take for plotting. Eg. .9 will use
       .05 and .95 as the lower and upper quantiles. Must be a float number
       between 0 and 1.
     digits: Number of decimals to display on metrics in the plot.
 
   Returns:
-      Plot of model fit.
+    Figure of the plot.
   """
-  if prediction.shape[1] != len(true_values):
-    raise ValueError(
-        "Predicted data and ground-truth data must have same length.")
-  upper_quantile = 1 - (1 - interval_mid_range) / 2
-  lower_quantile = (1 - interval_mid_range) / 2
-  upper_bound = jnp.quantile(a=prediction, q=upper_quantile, axis=0)
-  lower_bound = jnp.quantile(a=prediction, q=lower_quantile, axis=0)
-
-  r2, _ = arviz.r2_score(y_true=true_values, y_pred=prediction)
-  mape = 100 * metrics.mean_absolute_percentage_error(
-      y_true=true_values, y_pred=prediction.mean(axis=0))
-  fig, ax = plt.subplots(1, 1)
-  ax.plot(jnp.arange(true_values.shape[0]), true_values, c="grey", alpha=.9)
-  ax.plot(
-      jnp.arange(true_values.shape[0]),
-      prediction.mean(axis=0),
-      c="green",
-      alpha=.9)
-  ax.fill_between(
-      x=jnp.arange(true_values.shape[0]),
-      y1=lower_bound,
-      y2=upper_bound,
-      alpha=.35,
-      color="green")
-  ax.legend(["True KPI", "Predicted KPI"])
-  ax.yaxis.grid(color="gray", linestyle="dashed", alpha=0.3)
-  ax.xaxis.grid(color="gray", linestyle="dashed", alpha=0.3)
-  title = "\n".join([
-      "True and predicted KPI.",
-      "R2 = {r2:.{digits}f}".format(r2=r2, digits=digits),
-      "MAPE = {mape:.{digits}f}%".format(mape=mape, digits=digits)
-  ])
-  ax.title.set_text(title)
-  plt.close()
-  return fig
+  # TODO(): Allow to pass geo names for fit plots
+  if predictions.ndim == 3:  # Multiple plots for geo model
+    figure, axes = plt.subplots(predictions.shape[-1],
+                                figsize=(10, 5 * predictions.shape[-1]))
+    for i, ax in enumerate(axes):
+      _create_shaded_line_plot(predictions=predictions[..., i],
+                               target=target[..., i],
+                               axis=ax,
+                               title_prefix=f"Geo {i}:",
+                               interval_mid_range=interval_mid_range,
+                               digits=digits)
+  else:  # Single plot for national model
+    figure, ax = plt.subplots(1, 1)
+    _create_shaded_line_plot(predictions=predictions,
+                             target=target,
+                             axis=ax,
+                             interval_mid_range=interval_mid_range,
+                             digits=digits)
+  return figure
 
 
 def plot_model_fit(media_mix_model: lightweight_mmm.LightweightMMM,
@@ -437,8 +499,12 @@ def plot_model_fit(media_mix_model: lightweight_mmm.LightweightMMM,
   if target_scaler:
     posterior_pred = target_scaler.inverse_transform(posterior_pred)
     target_train = target_scaler.inverse_transform(target_train)
-  return _create_shaded_line_plot(posterior_pred, target_train,
-                                  interval_mid_range, digits)
+
+  return _call_fit_plotter(
+      predictions=posterior_pred,
+      target=target_train,
+      interval_mid_range=interval_mid_range,
+      digits=digits)
 
 
 def plot_out_of_sample_model_fit(out_of_sample_predictions: jnp.ndarray,
@@ -460,16 +526,18 @@ def plot_out_of_sample_model_fit(out_of_sample_predictions: jnp.ndarray,
   Returns:
     Plot of model fit.
   """
-  return _create_shaded_line_plot(out_of_sample_predictions,
-                                  out_of_sample_target, interval_mid_range,
-                                  digits)
+  return _call_fit_plotter(
+      predictions=out_of_sample_predictions,
+      target=out_of_sample_target,
+      interval_mid_range=interval_mid_range,
+      digits=digits)
 
 
 def plot_media_channel_posteriors(
     media_mix_model: lightweight_mmm.LightweightMMM,
     channel_names: Optional[Sequence[Any]] = None,
     quantiles: Sequence[float] = (0.05, 0.5, 0.95),
-    n_columns: int = 3) -> matplotlib.figure.Figure:
+    fig_size: Optional[Tuple[int, int]] = None) -> matplotlib.figure.Figure:
   """Plots the posterior distributions of estimated media channel effects.
 
   Model needs to be fit before calling this function to plot.
@@ -478,7 +546,9 @@ def plot_media_channel_posteriors(
     media_mix_model: Media mix model.
     channel_names: Names of media channels to be added to plot.
     quantiles: Quantiles to draw on the distribution.
-    n_columns: Number of columns of generated subplot.
+    fig_size: Size of the figure to plot as used by matplotlib. If not specified
+      it will be determined dynamically based on the number of media channels
+      and geos the model was trained on.
 
   Returns:
     Plot of posterior distributions.
@@ -486,20 +556,36 @@ def plot_media_channel_posteriors(
   if not hasattr(media_mix_model, "trace"):
     raise lightweight_mmm.NotFittedModelError(
         "Model needs to be fit first before attempting to plot its fit.")
+
   n_media_channels = np.shape(media_mix_model.trace["beta_media"])[1]
-  n_rows = (n_media_channels + n_columns - 1) // n_columns
+  n_geos = (
+      media_mix_model.media.shape[2] if media_mix_model.media.ndim == 3 else 1)
+
+  if not fig_size:
+    fig_size = (5 * n_geos, 3 * n_media_channels)
 
   media_channel_posteriors = media_mix_model.trace["beta_media"]
   if channel_names is None:
     channel_names = np.arange(np.shape(media_channel_posteriors)[1])
-  fig, axes = plt.subplots(n_rows, n_columns, figsize=(10, 10))
-  for index, ax in enumerate(axes.flatten()[:n_media_channels]):
-    ax = arviz.plot_kde(
-        media_channel_posteriors[:, index], quantiles=quantiles, ax=ax)
-    ax.set_xlabel(f"media channel {channel_names[index]}")
+  fig, axes = plt.subplots(
+      nrows=n_media_channels, ncols=n_geos, figsize=fig_size)
+  for channel_i, channel_axis in enumerate(axes):
+    if isinstance(channel_axis, np.ndarray):
+      for geo_i, geo_axis in enumerate(channel_axis):
+        geo_axis = arviz.plot_kde(
+            media_channel_posteriors[:, channel_i, geo_i],
+            quantiles=quantiles,
+            ax=geo_axis)
+        axis_label = f"media channel {channel_names[channel_i]} geo {geo_i}"
+        geo_axis.set_xlabel(axis_label)
+    else:
+      channel_axis = arviz.plot_kde(
+          media_channel_posteriors[:, channel_i],
+          quantiles=quantiles,
+          ax=channel_axis)
+      axis_label = f"media channel {channel_names[channel_i]}"
+      channel_axis.set_xlabel(axis_label)
 
-  for index, ax in enumerate(axes.flatten()[n_media_channels:]):
-    fig.delaxes(ax)
   fig.tight_layout()
   plt.close()
   return fig
@@ -530,6 +616,9 @@ def plot_bars_media_metrics(
   upper_quantile = 1 - (1 - interval_mid_range) / 2
   lower_quantile = (1 - interval_mid_range) / 2
 
+  if metric.ndim == 3:
+    metric = jnp.mean(metric, axis=-1)
+
   fig, ax = plt.subplots(1, 1)
   sns.barplot(data=metric, ci=None, ax=ax)
   quantile_bounds = np.quantile(
@@ -546,7 +635,9 @@ def plot_bars_media_metrics(
   ax.set_xticks(range(len(channel_names)))
   ax.set_xticklabels(channel_names, rotation=45)
   fig.suptitle(
-      f"Estimated media channel {metric_name}, error bars show {np.round(lower_quantile, 2)} - {np.round(upper_quantile, 2)} credibility interval"
+      f"Estimated media channel {metric_name}. \n Error bars show "
+      f"{np.round(lower_quantile, 2)} - {np.round(upper_quantile, 2)} "
+      "credibility interval."
   )
   plt.close()
   return fig
