@@ -18,8 +18,8 @@ import os
 
 from absl.testing import absltest
 from absl.testing import parameterized
-
 import jax.numpy as jnp
+import numpy as np
 
 from lightweight_mmm import lightweight_mmm
 from lightweight_mmm import utils
@@ -137,6 +137,65 @@ class UtilsTest(parameterized.TestCase):
           n_media_channels=n_media_channels,
           n_extra_features=n_extra_features)
 
+  def test_simulate_geo_data_has_right_shape(self):
+    data_size = 100
+    geos = 3
+    media_data, _, target, _ = utils.simulate_dummy_data(
+        data_size, 2, 2, geos=geos)
+    self.assertEqual(target.shape, (data_size, geos))
+    self.assertEqual(media_data.shape, (data_size, 2, geos))
+
+  def test_halfnormal_mean_and_scale(self):
+    mean = 1.
+    scale = utils.get_halfnormal_scale_from_mean(mean)
+    new_mean = utils.get_halfnormal_mean_from_scale(scale)
+    self.assertEqual(scale, mean * np.sqrt(np.pi) / np.sqrt(2))
+    self.assertEqual(mean, new_mean)
+
+  def test_beta_params_match(self):
+    a, b = 2., 3.
+    # Expected mean is 2 / 5.
+    mu = a / (a + b)
+    sigma = np.sqrt(a * b / ((a + b) ** 2 * (a + b + 1)))
+    ahat, bhat = utils.get_beta_params_from_mu_sigma(mu, sigma)
+    self.assertAlmostEqual(ahat / (ahat + bhat), 2 / 5)
+
+  def test_prior_posterior_distance_discrete(self):
+    p = jnp.array([0] * 2 + [1] * 3)
+    q = jnp.array([0] * 3 + [1] * 2 + [2] * 1)
+    ks = utils.distance_pior_posterior(p, q, method="KS", discrete=True)
+    js = utils.distance_pior_posterior(p, q, method="JS", discrete=True)
+    hell = utils.distance_pior_posterior(
+        p, q, method="Hellinger", discrete=True)
+    mindist = utils.distance_pior_posterior(p, q, method="min", discrete=True)
+    print(ks, js, hell, mindist)
+    self.assertAlmostEqual(ks, 1 / 6)
+    self.assertAlmostEqual(js, 0.283, 3)
+    self.assertAlmostEqual(hell, 0.325, 3)
+    self.assertAlmostEqual(mindist, 0.267, 3)
+
+  def test_prior_posterior_distance_continuous(self):
+    p = jnp.array([0] * 2 + [.5] * 3 + [1] * 2)
+    q = jnp.array([0] * 2 + [.5] * 4 + [1] * 2 + [1.5] * 3)
+    ks = utils.distance_pior_posterior(p, q, method="KS", discrete=False)
+    js = utils.distance_pior_posterior(p, q, method="JS", discrete=False)
+    hell = utils.distance_pior_posterior(
+        p, q, method="Hellinger", discrete=False)
+    mindist = utils.distance_pior_posterior(p, q, method="min", discrete=False)
+    print(ks, js, hell, mindist)
+    self.assertAlmostEqual(ks, 0.2727, 4)
+    self.assertAlmostEqual(js, 0.034, 3)
+    self.assertAlmostEqual(hell, 0.034, 3)
+    self.assertAlmostEqual(mindist, 0.041, 3)
+
+  def test_outlier_interpolation_straight_line(self):
+    x = np.arange(10) * 1.
+    x[3:5] += 10
+    x = jnp.array(x)
+    outlier_idx = jnp.array([3, 4])
+    new_x = utils.interpolate_outliers(x, outlier_idx)
+    self.assertTrue(all(np.equal(new_x[outlier_idx], [3, 4])))
+
 
 if __name__ == "__main__":
   absltest.main()
